//+------------------------------------------------------------------+
//|                    Holt double exponential smoothing - trend.mq4 |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers  4
#property indicator_color1   PaleVioletRed
#property indicator_color2   DimGray
#property indicator_color3   DeepSkyBlue
#property indicator_color4   DeepSkyBlue
#property indicator_width4   2
#property indicator_style2   STYLE_DOT


//
//
//
//
//

#import "dynamicZone.dll"
   double dzBuyP(double& sourceArray[],double probabiltyValue, int lookBack, int bars, int i, double precision);
   double dzSellP(double& sourceArray[],double probabiltyValue, int lookBack, int bars, int i, double precision);
#import

//
//
//
//
//

extern string TimeFrame              = "Current time frame";
extern double estimatePeriod         = 14.33;
extern double trendPeriod            = 10;
extern int    Price                  = PRICE_CLOSE;
extern int    DzLookBackBars         = 35;
extern double DzStartBuyProbability  = 0.1;
extern double DzStartSellProbability = 0.1;
extern bool   alertsOn               = false;
extern bool   alertsOnCurrent        = true;
extern bool   alertsMessage          = true;
extern bool   alertsSound            = false;
extern bool   alertsEmail            = false;
extern bool   Interpolate            = true;

//
//
//
//
//

double emaBuffer[];
double wrkBuffer[];
double bli[];
double sli[];
double zli[];
double trend[];

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(6);
   SetIndexBuffer(0,bli);
   SetIndexBuffer(1,zli);
   SetIndexBuffer(2,sli);
   SetIndexBuffer(3,wrkBuffer);
   SetIndexBuffer(4,emaBuffer);
   SetIndexBuffer(5,trend);
      estimatePeriod = MathMax(estimatePeriod,1);
      trendPeriod    = MathMax(trendPeriod,1);

      //
      //
      //
      //
      //
      
         indicatorFileName = WindowExpertName();
         calculateValue    = (TimeFrame=="calculateValue"); if (calculateValue) return(0);
         returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
         timeFrame         = stringToTimeFrame(TimeFrame);
      
      //
      //
      //
      //
      //
         
   IndicatorDigits(5);
   IndicatorShortName(timeFrameToString(timeFrame)+" Holt des trend ("+DoubleToStr(estimatePeriod,2)+","+DoubleToStr(trendPeriod,2)+")");
   return(0);
}
int deinit() { return(0); }

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { bli[0] = limit+1; return(0); }
         
   //
   //
   //
   //
   //
            
   if (calculateValue || timeFrame == Period())
   {
      double alpha = 2.0 /(1.0+estimatePeriod);
      double gamma = 2.0 /(1.0+trendPeriod);
      for(int i=limit; i>=0; i--)
      {
         double price = iMA(NULL,0,1,0,MODE_SMA,Price,i);
            if (i>=(Bars-3))
            {
               emaBuffer[i] = price;
               wrkBuffer[i] = price-emaBuffer[i+1];
               continue;
            }
         
            emaBuffer[i] = alpha*price+(1-alpha)*(emaBuffer[i+1]+wrkBuffer[i+1]);
            wrkBuffer[i] = gamma*(emaBuffer[i]-emaBuffer[i+1]) + (1-gamma)*wrkBuffer[i+1];
            bli[i]       = dzBuyP (wrkBuffer,DzStartBuyProbability,  DzLookBackBars, Bars, i, 0.0001);
            sli[i]       = dzSellP(wrkBuffer,DzStartSellProbability, DzLookBackBars, Bars, i, 0.0001);
            zli[i]       = dzSellP(wrkBuffer,0.5                   , DzLookBackBars, Bars, i, 0.0001);
            trend[i]     = trend[i+1];
               if (wrkBuffer[i]>zli[i]) trend[i] =  1;
               if (wrkBuffer[i]<zli[i]) trend[i] = -1;
      }
      manageAlerts();
      return(0);
   }
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         bli[i]       = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",estimatePeriod,trendPeriod,Price,DzLookBackBars,DzStartBuyProbability,DzStartSellProbability,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,0,y);
         zli[i]       = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",estimatePeriod,trendPeriod,Price,DzLookBackBars,DzStartBuyProbability,DzStartSellProbability,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,1,y);
         sli[i]       = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",estimatePeriod,trendPeriod,Price,DzLookBackBars,DzStartBuyProbability,DzStartSellProbability,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,2,y);
         wrkBuffer[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",estimatePeriod,trendPeriod,Price,DzLookBackBars,DzStartBuyProbability,DzStartSellProbability,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,3,y);
            
         //
         //
         //
         //
         //
      
         if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            for(int k = 1; k < n; k++)
            {
               bli[i+k]       = bli[i]       + (bli[i+n]       - bli[i]      )*k/n;
               zli[i+k]       = zli[i]       + (zli[i+n]       - zli[i]      )*k/n;
               sli[i+k]       = sli[i]       + (sli[i+n]       - sli[i]      )*k/n;
               wrkBuffer[i+k] = wrkBuffer[i] + (wrkBuffer[i+n] - wrkBuffer[i])*k/n;
            }               
   }
   return(0);

}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] == 1) doAlert(whichBar,"up");
         if (trend[whichBar] ==-1) doAlert(whichBar,"down");
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  timeFrameToString(Period())+Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" dz Holt trend changed to "+doWhat;
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"dz Holt double exponential"),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}